@@ -161,6 +161,7 @@ class NaPoleonBinanceFutureBot(object):
             return None
 
 
+
     def get_future_position_risk(self):
 
 
@@ -199,13 +200,46 @@ class NaPoleonBinanceFutureBot(object):
             return transfer_df[['date','asset','amount','type']]
         return transfer_df
 
+    def get_top_trader_long_short_ration(self,  asset, start_time, end_time, period):
+
+        startTime = int(start_time.timestamp() * 1000)
+        endTime = int(end_time.timestamp() * 1000)
+
+        BASE_URL = 'https://fapi.binance.com'
+        PATH = '/futures/data/topLongShortPositionRatio'
+
+
+        headers = {
+            'X-MBX-APIKEY': self.apiKey
+        }
+
+        params = {
+            'symbol': asset,
+            'period': period,
+            'startTime': startTime,
+            'endTime': endTime,
+            'limit':30
+
+        }
+        query_string = urllib.parse.urlencode(params)
+        params['signature'] = hmac.new(self.secret.encode('utf-8'), query_string.encode('utf-8'),
+                                       hashlib.sha256).hexdigest()
+        url = urljoin(BASE_URL, PATH)
+        print(url)
+        r = requests.get(url, headers=headers, params=params)
+        dataSet = r.json()
+        ls_ratio_df = pd.DataFrame(dataSet)
+        ls_ratio_df.index = pd.to_datetime(ls_ratio_df['timestamp'], unit='ms')
+
+        return ls_ratio_df
+
     def get_coin_futures_transfer_history(self, asset, start_time, end_time):
 
         startTime = int(start_time.timestamp() * 1000)
         endTime = int(end_time.timestamp() * 1000)
 
         BASE_URL = 'https://api.binance.com'
-        PATH = '/sapi/v1/futures/transfer'
+        PATH = '/sapi/v1/futures/data/topLongShortPositionRatio'
 
 
         headers = {
@@ -239,6 +273,7 @@ class NaPoleonBinanceFutureBot(object):
         else :
             return None
 
+
     def get_balance_low_level(self, request_date):
         BASE_URL = 'https://api.binance.com'
         PATH = '/api/v3/account'
@@ -263,6 +298,8 @@ class NaPoleonBinanceFutureBot(object):
         dataSet = r.json()
         print(dataSet)
 
+
+
     def get_balance(self):
         # Recupérer la balance
         balance = self.exchange.fetchTotalBalance(params={})
