@@ -406,7 +406,7 @@ class NaPoleonXConnector(object):
 
     url_auth = 'https://api.napoleonx.ai/auth-service/oauth/token'
     url = 'https://api.napoleonx.ai/quote-service/v1/eod-quote/filter'
-    signal_url = 'https://api.napoleonx.ai/position-service/v1/signal-computation/filter'
+    signal_url = 'https://api.napoleonx.ai/signal-service/v1/signal-computation/filter'
     url_product = 'https://api.napoleonx.ai/product-service/v1/product/'
     data = {
         'grant_type': 'password',
