@@ -748,6 +748,16 @@ class RealTimeSignalEnsemblingParalleLauncher():
         return y_next_pred_discrete
 
 
+    def daily_apply_single_indice_last_signal(self):
+        signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
+        prediction_index = self.day_signal_df.shape[0] - 1
+        starting_train = prediction_index - self.daily_ia_lookback_window
+        stopping_train = prediction_index
+        local_signal_df = self.day_signal_df[signal_columns]
+        y_next_pred = local_signal_df.values[prediction_index, local_signal_df.columns.get_loc(self.selected_indice)]
+        print('day discrete prediction ' + str(y_next_pred))
+        return y_next_pred
+
 
     def daily_apply_standard_mean_discretization_last_signal(self, method='find_best_three_states_discretization_lo'):
         signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
@@ -1264,7 +1274,10 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print(f'last daily alphas signal {last_day_signals}')
         print('discretizing the signal')
         if  self.day_signal_np.shape[0]> self.daily_ia_lookback_window + 1:
-            discrete_signals = self.daily_apply_standard_mean_discretization_last_signal()
+            if self.selected_indice is None:
+                discrete_signals = self.daily_apply_standard_mean_discretization_last_signal()
+            else :
+                discrete_signals = self.daily_apply_single_indice_last_signal()
         else :
             print('trouble : not enough daily histo for proper quantization')
             return last_day_ts,np.nanmean(last_day_signals)
