@@ -30,7 +30,7 @@ __all__ = ['minDrawdown','ERC', 'HRP', 'IVP', 'MDP', 'MVP', 'MVP_uc', 'rolling_a
 # =========================================================================== #
 
 
-def ERC(X, w0=None, up_bound=1., low_bound=0.):
+def ERC(X, w0=None, up_bound=1., low_bound=0., transaction_costs = None):
     r""" Get weights of Equal Risk Contribution portfolio allocation.
 
     Notes
@@ -238,7 +238,7 @@ def _get_IVP(mat_cov):
     return ivp
 ################################
 
-def HRC(X, low_bound=0., up_bound=1.0, nb_clusters = 5):
+def HRC(X, low_bound=0., up_bound=1.0, nb_clusters = 5, transaction_costs = None):
     if not isinstance(X, pd.DataFrame):
         X = pd.DataFrame(X)
 
@@ -325,7 +325,7 @@ def _compute_hrc_allocation(covar, clusters, dim, Z):
     return assets_weights
 
 
-def HRP(X, method='single', metric='euclidean', low_bound=0., up_bound=1.0):
+def HRP(X, method='single', metric='euclidean', low_bound=0., up_bound=1.0, transaction_costs = None):
     r""" Get weights of the Hierarchical Risk Parity allocation.
 
     Notes
@@ -380,7 +380,7 @@ def HRP(X, method='single', metric='euclidean', low_bound=0., up_bound=1.0):
     return _normalize(w, up_bound=up_bound, low_bound=low_bound)
 
 
-def minSharpe(X, w0=None, normalize=False, low_bound=0., up_bound=1.0):
+def minSharpe(X, w0=None, normalize=False, low_bound=0., up_bound=1.0, transaction_costs = None):
     T, N = X.shape
     SIGMA = np.cov(X, rowvar=False)
     if N == 0:
@@ -408,16 +408,18 @@ def minSharpe(X, w0=None, normalize=False, low_bound=0., up_bound=1.0):
     return result.x.reshape([N, 1])
 
 
-def maxPerf(X,  w0=None, normalize=False, low_bound=0., up_bound=1.0):
+def maxPerf(X,  w0=None, normalize=False, low_bound=0., up_bound=1.0, transaction_costs = None):
     T, N = X.shape
     up_bound = max(up_bound, 1 / N)
 
-    def f_performance(w, X):
-        cumulated_perf = np.cumprod(X @ w + 1)
+    def f_performance(w, X, t):
+        normalized_transactions = t/abs(sum(t))
+        returns_over_time = X @ w + 1
+        cumulated_perf = np.cumprod(returns_over_time)
         final_cumulated_perf = cumulated_perf[-1]
         return final_cumulated_perf
 
-    toOptimize = lambda w : f_performance(w,X)
+    toOptimize = lambda w : f_performance(w,X,transaction_costs)
 
     # Set inital weights
     if w0 is None:
@@ -435,7 +437,7 @@ def maxPerf(X,  w0=None, normalize=False, low_bound=0., up_bound=1.0):
 
     return result.x.reshape([N, 1])
 
-def minDrawdown(X,  w0=None, normalize=False, low_bound=0., up_bound=1.0):
+def minDrawdown(X,  w0=None, normalize=False, low_bound=0., up_bound=1.0, transaction_costs = None):
     T, N = X.shape
     SIGMA = np.cov(X, rowvar=False)
     up_bound = max(up_bound, 1 / N)
@@ -465,7 +467,7 @@ def minDrawdown(X,  w0=None, normalize=False, low_bound=0., up_bound=1.0):
 #                        Inverse Variance Portfolio                           #
 # =========================================================================== #
 
-def IVP(X, normalize=False, low_bound=0., up_bound=1.0):
+def IVP(X, normalize=False, low_bound=0., up_bound=1.0, transaction_costs = None):
     r""" Get weights of the Inverse Variance Portfolio allocation.
 
     Notes
@@ -519,7 +521,7 @@ def IVP(X, normalize=False, low_bound=0., up_bound=1.0):
 # =========================================================================== #
 
 
-def MVP(X, normalize=False):
+def MVP(X, normalize=False, transaction_costs = None):
     r""" Get weights of the Minimum Variance Portfolio allocation.
 
     Notes
@@ -577,7 +579,7 @@ def MVP(X, normalize=False):
     return w
 
 
-def MVP_uc(X, w0=None, up_bound=1., low_bound=0.):
+def MVP_uc(X, w0=None, up_bound=1., low_bound=0., transaction_costs = None):
     r""" Get weights of the Minimum Variance Portfolio under constraints.
 
     Notes
@@ -640,7 +642,7 @@ def MVP_uc(X, w0=None, up_bound=1., low_bound=0.):
 # =========================================================================== #
 
 
-def MDP(X, w0=None, up_bound=1., low_bound=0.):
+def MDP(X, w0=None, up_bound=1., low_bound=0., transaction_costs = None):
     r""" Get weights of Maximum Diversified Portfolio allocation.
 
     Notes
@@ -758,7 +760,7 @@ def rolling_forward_looking_allocation(f, X, n=252, s=63, ret=True, drift=True,
 
     return portfolio, w_mat
 
-def rolling_allocation(f, X, n=252, s=63, ret=True, drift=True, filtering_threshold = 0.7, **kwargs):
+def rolling_allocation(f, X, n=252, s=63, ret=True, drift=True, filtering_threshold = 0.7, transaction_costs = None, **kwargs):
     r""" Roll an algorithm of portfolio allocation.
 
     Notes
@@ -820,11 +822,14 @@ def rolling_allocation(f, X, n=252, s=63, ret=True, drift=True, filtering_thresh
         sub_X = X_.loc[slice_n].copy()
         assets = list(X.columns[sub_X.apply(allocation_process)])
         sub_X = sub_X.fillna(method='bfill')
+
+        assets_iloc = [sub_X.columns.get_loc(ass) for ass in assets]
+
         # Compute weights
         if len(assets) == 1:
             w = np.array([[1.]])
         else:
-            w = f(sub_X.loc[:, assets].values, **kwargs)
+            w = f(sub_X.loc[:, assets].values, transaction_costs = transaction_costs[assets_iloc], **kwargs)
 
         if w.flatten().sum(axis = 0 )>1:
             w=w/w.sum(axis=0)
