@@ -40,8 +40,49 @@ def generate_anova_like(signal_action_types, mixing_types):
             })
     return parameters
 
+def generate_past_perf_mod(looks, threshs,lows,highs):
+    parameters = []
+    for look in looks:
+        for thresh in threshs:
+            for low in lows:
+                for high in highs:
+                    parameters.append((look, thresh, high, low))
+    return parameters
 
+def generate_volume_strength(lags, middles, low_levs, high_levs, contravariants, colmuns, signals):
+    parameters = []
+    for lag in lags:
+        for middle in middles:
+            for low_lev in low_levs:
+                for high_lev in high_levs:
+                    for contravariant in contravariants:
+                        for colmun in colmuns:
+                            for signal in signals:
+                                parameters.append((lag, middle, low_lev, high_lev, contravariant,colmun,signal))
+    return parameters
 
+def generate_volume_filter_rsi(lags, middles, low_levs, high_levs, contravariants, colmuns):
+    parameters = []
+    for lag in lags:
+        for middle in middles:
+            for low_lev in low_levs:
+                for high_lev in high_levs:
+                    for contravariant in contravariants:
+                        for colmun in colmuns:
+                            parameters.append((lag, middle, low_lev, high_lev, contravariant,colmun))
+    return parameters
+
+def generate_volume_filter_lead_lag(leads, lags, low_levs, high_levs, percentage_longs, coliumns):
+    parameters = []
+    for lead in leads:
+        for lag in lags:
+            for low_lev in low_levs:
+                for high_lev in high_levs:
+                    for percentage_long in percentage_longs:
+                        for column in coliumns:
+                            if lead < lag:
+                                parameters.append((lead,lag,low_lev,high_lev, percentage_long, column))
+    return parameters
 
 def generate_lead_lag(lookback_windows, lead_lags, contravariants, ):
     parameters = []
