@@ -26,6 +26,10 @@ import seaborn as sns
 import warnings
 warnings.simplefilter("ignore")
 
+import os
+
+
+from napoleontoolbox.file_saver import dropbox_file_saver
 
 # Utils
 def convert_unix(ts):
@@ -220,7 +224,7 @@ class BitMEX:
     def __init__(self, api_key, private_key, *args):
         self.api_key = api_key
         self.p_key = private_key
-        self.KeyAuth = APIKeyAuthWithExpires(api_key, pkey)
+        self.KeyAuth = APIKeyAuthWithExpires(api_key, self.p_key)
 
     # Where optype = position, stats, funding, orderBook, trade, /user/executionHistory
     # Format we need for startdate, enddate = 2019-04-25T00:00:00.545Z
@@ -463,7 +467,9 @@ def numba_compute(arr, d):
     names = ['lastPxBTC', 'lastPxETH', 'traded_xbt', 'traded_eth', 'size_xbt', 'size_eth', 'sens_trade_xbt',
              'sens_trade_eth', 'markets_BTC', 'markets_ETH']
     arr, d = _merge_update(arr, d, arrays, names)
-    arrays = get_fees(df, d)
+    #arrays = get_fees(df, d)
+    arrays = get_fees(arr, d)
+
     names = ['funding_fees_xbt_btc', 'funding_fees_eth_btc', 'funding_fees_xbt_usd', 'funding_fees_eth_usd',
              'trading_fees_xbt_btc', 'trading_fees_xbt_usd', 'trading_fees_eth_btc', 'trading_fees_eth_usd']
     arr, d = _merge_update(arr, d, arrays, names)
@@ -513,4 +519,247 @@ def get_fees(arr, d):
         fold_a = []
         for _ in fold:
             fold_a.append(np.array(_))
-        return fold_a
\ No newline at end of file
+        return fold_a
+
+def fetch_all_necessary_data(public_key = None, private_key= None, start_time= None, end_time= None, save_to_dropbox = False, dropbox_token = None, local_root_directory = './'):
+
+    dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=dropbox_token, dropbox_backup=True)
+
+    start_time_stub = start_time.strftime('%d_%b_%Y')
+    end_time_stub = end_time.strftime('%d_%b_%Y')
+    filename_stub = f'{public_key}_{start_time_stub}_{end_time_stub}_Bitmex'
+    #filename_stub = f'{inv_year_dictionary[year]}_{inv_month_dictionary[month]}_{inv_market_dictionary[market]}'
+
+
+    report_pkl_file_name = f'{filename_stub}_report.pkl'
+    trade_pkl_file_name = f'{filename_stub}_trades.pkl'
+    ohlc_eth_bm_pkl_file_name = f'{filename_stub}_ohlc_eth_bm.pkl'
+    ohlc_btc_bm_pkl_file_name = f'{filename_stub}_ohlc_btc_bm.pkl'
+    walletHist_pkl_file_name = f'{filename_stub}_walletHist.pkl'
+    ohlc_btc_cc_pkl_file_name = f'{filename_stub}_ohlc_btc_cc.pkl'
+    ohlc_eth_cc_pkl_file_name = f'{filename_stub}_ohlc_eth_cc.pkl'
+
+    client = bitmex.bitmex(test=False, config=None, api_key=public_key, api_secret=private_key)
+    if isinstance(start_time,dt.datetime) and isinstance(end_time,dt.datetime):
+        start_time = start_time.strftime('%Y-%m-%dT%H:%M')
+        end_time = end_time.strftime('%Y-%m-%dT%H:%M')
+
+
+    ce, te = parser.parse(start_time).replace(tzinfo=pytz.UTC), parser.parse(end_time).replace(tzinfo=pytz.UTC)
+
+    ce = ce - dt.timedelta(hours=8)
+    trades = pd.DataFrame()
+    print('Accessing Trade History...')
+    while ce < te:
+        print(f'requesting trades between {ce} and {te}')
+        tmp = pd.DataFrame(client.Execution.Execution_getTradeHistory(startTime=ce, count=500).result()[0]).sort_values(
+            by='timestamp', ascending=True)
+        trades = trades.append(tmp)
+        time.sleep(4)
+        ce = tmp.timestamp.iloc[-1]
+    trades = trades[trades.timestamp <= te].copy()
+    trades.execComm = - trades.execComm  # logique comptable
+    trades = trades.sort_values(by='timestamp', ascending=True).drop_duplicates()
+    print('Done.')
+
+    # Get wallet history for a period of time
+    ce, te = parser.parse(start_time).replace(tzinfo=pytz.UTC), parser.parse(end_time).replace(tzinfo=pytz.UTC)
+    ce = ce - dt.timedelta(hours=8)
+    walletHist = pd.DataFrame()
+    print('Accessing wallet history...')
+    while ce < te:
+        print(f'requesting wallet history between {ce} and {te}')
+        tmp = pd.DataFrame(client.User.User_getWalletHistory(currency='XBt').result()[0]).sort_values(by='timestamp',
+                                                                                                      ascending=True)
+        walletHist = walletHist.append(tmp)
+        time.sleep(3.5)
+        ce = tmp.timestamp.iloc[-1]
+    walletHist = walletHist[walletHist.timestamp <= te].copy()
+    print('Wallet History retrieved.')
+
+    # Récupérer les OHLC hourly sur Cryptocompare depuis startdate
+    cc = CryptoCompare('')
+    ohlc_btc_cc = cc.get('OHLC', 'BTC', start_time, end_time, precision='hour')
+    ohlc_eth_cc = cc.get('OHLC', 'ETH', start_time, end_time, precision='hour')
+    print("OHLC Cryptocompare retrieved.")
+
+    # Récupérer les OHLC hourly sur BM depuis startdate
+
+    bm = BitMEX(public_key, private_key)
+    ohlc_btc_bm = bm.get('OHLC', currency='XBT', startdate=start_time, enddate=end_time, precision='1h')
+    ohlc_eth_bm = bm.get('OHLC', currency='ETH', startdate=start_time, enddate=end_time, precision='1h')
+    print('OHLC BM retrieved.')
+
+    if save_to_dropbox :
+        full_path = local_root_directory + trade_pkl_file_name
+        trades.to_pickle(full_path)
+        print(f'uploading to dropbox {full_path}')
+        dbx.upload(fullname=full_path, folder='', subfolder='', name=trade_pkl_file_name, overwrite=True)
+        os.remove(full_path)
+
+        full_path = local_root_directory + ohlc_eth_bm_pkl_file_name
+        ohlc_eth_bm.to_pickle(full_path)
+        print(f'uploading to dropbox {full_path}')
+        dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_eth_bm_pkl_file_name, overwrite=True)
+        os.remove(full_path)
+
+        full_path = local_root_directory + ohlc_btc_bm_pkl_file_name
+        ohlc_btc_bm.to_pickle(full_path)
+        print(f'uploading to dropbox {full_path}')
+        dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_btc_bm_pkl_file_name, overwrite=True)
+        os.remove(full_path)
+
+        full_path = local_root_directory + walletHist_pkl_file_name
+        walletHist.to_pickle(full_path)
+        print(f'uploading to dropbox {full_path}')
+        dbx.upload(fullname=full_path, folder='', subfolder='', name=walletHist_pkl_file_name, overwrite=True)
+        os.remove(full_path)
+
+        full_path = './' + ohlc_btc_cc_pkl_file_name
+        ohlc_btc_cc.to_pickle(full_path)
+        print(f'uploading to dropbox {full_path}')
+        dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_btc_cc_pkl_file_name, overwrite=True)
+        os.remove(full_path)
+
+        full_path = local_root_directory + ohlc_eth_cc_pkl_file_name
+        ohlc_eth_cc.to_pickle(full_path)
+        print(f'uploading to dropbox {full_path}')
+        dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_eth_cc_pkl_file_name, overwrite=True)
+        os.remove(full_path)
+
+    # ohlc_btc_bm
+    x = ohlc_btc_bm
+    x = x.set_index('timestamp')
+    x.index = x.index.map(parser.parse)
+    x.index = x.index.tz_localize(None)
+    x.columns = [y + '_btc_bm' for y in x.columns]
+    ohlc_btc_bm = x
+
+    # ohlc_eth_bm
+    x = ohlc_eth_bm
+    x = x.set_index('timestamp')
+    x.index = x.index.map(parser.parse)
+    x.index = x.index.tz_localize(None)
+    x.columns = [y + '_eth_bm' for y in x.columns]
+    ohlc_eth_bm = x
+
+    # ohlc_btc_cc
+    x = ohlc_btc_cc
+    x.index = x.index.map(parser.parse)
+    x.index = x.index.tz_localize(None)
+    x.columns = [y + '_btc_cc' for y in x.columns]
+    ohlc_btc_cc = x
+
+    # ohlc_eth_cc
+    x = ohlc_eth_cc
+    x.index = x.index.map(parser.parse)
+    x.index = x.index.tz_localize(None)
+    x.columns = [y + '_eth_cc' for y in x.columns]
+    ohlc_eth_cc = x
+
+    ohlc_eth_cc = ohlc_eth_cc.sort_index(ascending=True).drop_duplicates()
+    ohlc_btc_cc = ohlc_btc_cc.sort_index(ascending=True).drop_duplicates()
+    ohlc_btc_bm = ohlc_btc_bm.sort_index(ascending=True).drop_duplicates()
+    ohlc_eth_bm = ohlc_eth_bm.sort_index(ascending=True).drop_duplicates()
+
+    ##############################
+    ##############################
+    ##########
+    ########## computing, grouping
+    ##########
+    ##############################
+    ##############################
+
+    start = time.time()
+    df, d = trades_to_arr(trades)
+    df, d = numba_compute(df, d)
+    print('Done for', trades.shape[0], 'trades in ', time.time() - start, 'seconds.')
+
+    # Now: Groupby
+    df = pd.DataFrame(df, columns=[i for i in d.keys()])
+    df['funding_fees_xbt_usd'] = df.funding_fees_xbt_btc * df.lastPxBTC
+    df['funding_fees_eth_usd'] = df.funding_fees_eth_btc * df.lastPxBTC
+    df['trading_fees_eth_usd'] = df.trading_fees_eth_btc * df.lastPxBTC
+    df['trading_fees_xbt_usd'] = df.trading_fees_xbt_btc * df.lastPxBTC
+    df.transactTime = df.transactTime.apply(lambda x: dt.datetime.utcfromtimestamp(x / 1000))
+    df['dt_floor'] = df.transactTime.dt.floor('H')
+    df[
+        'total_fees_usd'] = df.trading_fees_xbt_usd + df.trading_fees_eth_usd + df.funding_fees_xbt_usd + df.funding_fees_eth_usd
+
+    print('Grouping by...')
+    td = df[['dt_floor', 'traded_eth', 'traded_xbt', 'size_eth', 'size_xbt', 'sens_trade_eth', 'sens_trade_xbt',
+             'funding_fees_xbt_usd', 'funding_fees_eth_usd',
+             'trading_fees_xbt_btc', 'trading_fees_xbt_usd', 'trading_fees_eth_btc', 'trading_fees_eth_usd',
+             'total_fees_usd']].groupby('dt_floor').sum()
+
+    td.index = td.index.tz_localize(None)
+    td['avg_eth_price'] = td.size_eth / abs(td.traded_eth)
+    td['avg_btc_price'] = td.size_xbt / abs(td.traded_xbt)
+
+    # Join & compute prices for cryptocompare
+    td = td.join(ohlc_eth_cc['open_eth_cc']).join(ohlc_btc_cc['open_btc_cc'])
+    td['slippage_eth_price_cc'] = (td.avg_eth_price / td.open_eth_cc - 1) * np.sign(td.sens_trade_eth)
+    td['slippage_eth_cash_cc'] = td.slippage_eth_price_cc * abs(
+        td.traded_eth)  # / 1000000 * td.open_btc_cc * td.open_eth_cc
+    td['slippage_btc_price_cc'] = (td.avg_btc_price / td.open_btc_cc - 1) * np.sign(td.sens_trade_xbt)
+    td['slippage_btc_cash_cc'] = td.slippage_btc_price_cc * abs(td.traded_xbt)
+    # slippage en 100e de millionnième de eth = ratio * cours open * qty * 100
+    # slippage en 100e de millionnième de btc = ratio * 1Mo/ cours * 100 * qty
+
+    # Join & compute slippage for bitmex
+    td = td.join(ohlc_eth_bm['open_eth_bm']).join(ohlc_btc_bm['open_btc_bm'])
+    td['slippage_eth_price_bm'] = (td.avg_eth_price / td.open_eth_bm - 1) * np.sign(td.sens_trade_eth)
+    td['slippage_eth_cash_bm'] = td.slippage_eth_price_bm * abs(
+        td.traded_eth)  # / 1000000 * td.open_btc_bm * td.open_eth_bm
+    td['slippage_btc_price_bm'] = (td.avg_btc_price / td.open_btc_bm - 1) * np.sign(td.sens_trade_xbt)
+    td['slippage_btc_cash_bm'] = td.slippage_btc_price_bm * abs(td.traded_xbt)
+    print('All done.')
+
+    walletHist = walletHist.set_index('timestamp')
+    walletHist.index = walletHist.index.map(lambda x: x.floor('60s'))
+    walletHist = walletHist[~walletHist.index.duplicated(keep='first')].sort_index(ascending=True)
+    walletHist.index = walletHist.index.tz_localize(None)
+    walletHist['walletBalance_xbt'] = walletHist.walletBalance / 100000000
+    walletHist = walletHist.join(ohlc_btc_cc['open_btc_cc'])
+    walletHist['walletBalance_usd'] = walletHist.walletBalance_xbt * walletHist.open_btc_cc
+
+    # Join to td
+    td_h = td.join(walletHist.walletBalance_usd.dropna(), rsuffix='_fromWH')
+    td_h.walletBalance_usd = td_h.walletBalance_usd.fillna(method='ffill')
+
+    td_h['traded_eth_usd_a'] = td_h.traded_eth * td_h.avg_eth_price * 0.000001 * td_h.open_btc_bm
+    td_h['traded_xbt_usd_a'] = td_h.traded_xbt / td_h.avg_btc_price
+    # Usd value = xbt spot * eht usd price * bitcoin multiplier ( 0.000001)
+
+    x = td_h[['traded_eth', 'avg_eth_price', 'traded_eth_usd_a', 'sens_trade_eth', 'trading_fees_eth_usd',
+              'funding_fees_eth_usd', 'open_eth_cc', 'open_eth_bm',
+              'traded_xbt', 'avg_btc_price', 'traded_xbt_usd_a', 'sens_trade_xbt', 'trading_fees_xbt_usd',
+              'funding_fees_xbt_usd', 'open_btc_cc', 'open_btc_bm', 'walletBalance_usd',
+              'trading_fees_eth_usd', 'funding_fees_eth_usd', 'slippage_eth_price_cc', 'slippage_eth_cash_cc',
+              'trading_fees_xbt_usd', 'funding_fees_xbt_usd', 'slippage_btc_price_cc', 'slippage_btc_cash_cc',
+              'trading_fees_eth_usd', 'funding_fees_eth_usd', 'slippage_eth_price_bm', 'slippage_eth_cash_bm',
+              'trading_fees_xbt_usd', 'funding_fees_xbt_usd', 'slippage_btc_price_bm', 'slippage_btc_cash_bm']]
+
+    x = x.rename_axis('timestamp')
+    x.columns = ['quantity_traded_eth', 'avg_traded_price_eth', 'amount_traded_usd_eth', 'n_trades_eth',
+                 'trading_fees_usd_eth', 'funding_fees_usd_eth',
+                 'open_eth_cc', 'open_eth_exchange',
+                 'quantity_traded_btc', 'avg_traded_price_btc', 'amount_traded_usd_btc', 'n_trades_btc',
+                 'trading_fees_usd_btc', 'funding_fees_usd_btc',
+                 'open_btc_cc', 'open_btc_exchange',
+                 'wallet_amount_usd',
+                 'trading_fees_usd_eth', 'funding_fees_usd_eth', 'slippage_eth_price_cc', 'slippage_eth_cash_cc',
+                 'trading_fees_usd_btc', 'funding_fees_usd_btc', 'slippage_btc_price_cc', 'slippage_btc_cash_cc',
+                 'trading_fees_usd_eth', 'funding_fees_usd_eth', 'slippage_eth_price_bm', 'slippage_eth_cash_bm',
+                 'trading_fees_usd_btc', 'funding_fees_usd_btc', 'slippage_btc_price_bm', 'slippage_btc_cash_bm']
+
+    x = x.fillna(0).round(decimals=5)
+
+    full_path = local_root_directory + report_pkl_file_name
+    x.to_pickle(full_path)
+    print(f'uploading to dropbox {full_path}')
+    dbx.upload(fullname=full_path, folder='', subfolder='', name=report_pkl_file_name, overwrite=True)
+    os.remove(full_path)
+
+    return x
+
